import multiprocessing
import os
from datetime import datetime

import chardet
import pandas as pd

pd.options.mode.copy_on_write = True


# 获取文件编码
def detect_file_encoding(filename):
    # 读取文件的前1000个字节（足够用于大多数编码检测）
    with open(filename, 'rb') as f:
        rawdata = f.read(1000)
    result = chardet.detect(rawdata)
    encoding = result['encoding']

    if encoding is None:
        encoding = 'gb18030'

    if encoding and encoding.lower() == 'gb2312' or encoding.lower().startswith("windows"):
        encoding = 'gb18030'
    return encoding


# 读取数据到df
def read_file_to_df(file_path, read_cols=list(), header=0):
    df = pd.DataFrame()
    if str(file_path).lower().endswith("csv") or str(file_path).lower().endswith("gz"):
        encoding = detect_file_encoding(file_path)
        end_with_gz = str(file_path).lower().endswith("gz")
        if read_cols:
            if end_with_gz:
                df = pd.read_csv(file_path, encoding=encoding, usecols=read_cols, compression='gzip', header=header)
            else:
                df = pd.read_csv(file_path, encoding=encoding, usecols=read_cols, header=header, on_bad_lines='warn')
        else:

            if end_with_gz:
                df = pd.read_csv(file_path, encoding=encoding, compression='gzip', header=header)
            else:
                df = pd.read_csv(file_path, encoding=encoding, header=header, on_bad_lines='warn')

    else:
        xls = pd.ExcelFile(file_path)
        # 获取所有的sheet名称
        sheet_names = xls.sheet_names
        for sheet in sheet_names:
            if read_cols:
                df = pd.concat([df, pd.read_excel(xls, sheet_name=sheet, header=header, usecols=read_cols)])
            else:
                df = pd.concat([df, pd.read_excel(xls, sheet_name=sheet, header=header)])

    return df


def __build_directory_dict(directory_dict, path, filter_types=None):
    # 遍历目录下的所有项
    for item in os.listdir(path):
        item_path = os.path.join(path, item)
        if os.path.isdir(item_path):
            __build_directory_dict(directory_dict, item_path, filter_types=filter_types)
        elif os.path.isfile(item_path):
            if path not in directory_dict:
                directory_dict[path] = []

            if filter_types is None or len(filter_types) == 0:
                directory_dict[path].append(item_path)
            elif str(item_path).split(".")[-1] in filter_types:
                if str(item_path).count("~$") == 0:
                    directory_dict[path].append(item_path)


def read_excel_files(read_path):
    directory_dict = {}
    __build_directory_dict(directory_dict, read_path, filter_types=['xls', 'xlsx', 'csv', 'gz'])

    return [path for paths in directory_dict.values() for path in paths if path]


def read_and_save(file_path, read_dir, save_dir, time_col, time_during, select_cols_str):
    base_name = os.path.basename(file_path)
    print(f'{datetime.now()}: {base_name} 开始执行')
    begin = datetime.now()
    select_cols = list()
    if select_cols_str:
        select_cols = [i for i in select_cols_str.split(",") if i]

    if select_cols:
        if time_col not in select_cols:
            select_cols.insert(0, time_col)
        df = read_file_to_df(file_path, read_cols=select_cols)
    else:
        df = read_file_to_df(file_path)

    print(f'{base_name} : {df.shape}')
    df[time_col] = pd.to_datetime(df[time_col], errors='coerce')
    df = df[(df[time_col] >= datetime.strptime(time_during.split(",")[0], '%Y-%m-%d %H:%M:%S')) &
            (df[time_col] < datetime.strptime(time_during.split(",")[1], '%Y-%m-%d %H:%M:%S'))]

    save_path = file_path.replace(read_dir, save_dir)
    os.makedirs(os.path.dirname(save_path), exist_ok=True)
    df.to_csv(save_path, index=False, encoding='utf-8')
    print(f"{datetime.now()}: 执行{os.path.basename(file_path)}结束,耗时{datetime.now() - begin}")


if __name__ == '__main__':
    read_dir = r'/data/download/collection_data/1进行中/新艾里风电场-吉林-大唐/清理数据/WOF043600007-WOB000004_XAL1219秒级/second'
    save_dir = r'/data/download/collection_data/1进行中/新艾里风电场-吉林-大唐/清理数据/WOF043600007-WOB000004_XAL1219秒级/8-9-10-month-data'
    time_col = 'time_stamp'
    time_during = '2024-08-01 00:00:00,2024-11-01 00:00:00'
    select_cols_str = 'time_stamp,active_power,wind_velocity,pitch_angle_blade_1,yaw_error1'

    all_excel_files = read_excel_files(read_dir)

    with multiprocessing.Pool(processes=10) as pool:
        pool.starmap(read_and_save,
                     [(file_path, read_dir, save_dir, time_col, time_during, select_cols_str) for file_path in
                      all_excel_files])
